
/* Chrysalide - Outil d'analyse de fichiers binaires
 * demangler.c - équivalent Python du fichier "mangling/demangler.c"
 *
 * Copyright (C) 2018 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "demangler.h"


#include <pygobject.h>


#include <mangling/demangler.h>


#include "../helpers.h"



/* Tente de décoder une chaîne de caractères donnée en type. */
static PyObject *py_compiler_demangler_decode_type(PyObject *, PyObject *);

/* Tente de décoder une chaîne de caractères donnée en routine. */
static PyObject *py_compiler_demangler_decode_routine(PyObject *, PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = décodeur à solliciter pour l'opération.               *
*                args = chaîne de caractères à décoder.                       *
*                                                                             *
*  Description : Tente de décoder une chaîne de caractères donnée en type.    *
*                                                                             *
*  Retour      : Instance obtenue ou None en cas d'échec.                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_compiler_demangler_decode_type(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Désignation à retourner     */
    const char *desc;                       /* Description à traiter       */
    int ret;                                /* Bilan de lecture des args.  */
    GCompDemangler *demangler;              /* Décodeur mis en place       */
    GDataType *type;                        /* Type de données obtenu      */

    ret = PyArg_ParseTuple(args, "s", &desc);
    if (!ret) return NULL;

    demangler = G_COMP_DEMANGLER(pygobject_get(self));

    type = g_compiler_demangler_decode_type(demangler, desc);

    if (type != NULL)
    {
        result = pygobject_new(G_OBJECT(type));
        Py_INCREF(result);

        g_object_unref(G_OBJECT(type));

    }
    else
    {
        result = Py_None;
        Py_INCREF(result);
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self = décodeur à solliciter pour l'opération.               *
*                args = chaîne de caractères à décoder.                       *
*                                                                             *
*  Description : Tente de décoder une chaîne de caractères donnée en routine. *
*                                                                             *
*  Retour      : Instance obtenue ou None en cas d'échec.                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_compiler_demangler_decode_routine(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Désignation à retourner     */
    const char *desc;                       /* Description à traiter       */
    int ret;                                /* Bilan de lecture des args.  */
    GCompDemangler *demangler;              /* Décodeur mis en place       */
    GBinRoutine *routine;                   /* Routine obtenue             */

    ret = PyArg_ParseTuple(args, "s", &desc);
    if (!ret) return NULL;

    demangler = G_COMP_DEMANGLER(pygobject_get(self));

    routine = g_compiler_demangler_decode_routine(demangler, desc);

    if (routine != NULL)
    {
        result = pygobject_new(G_OBJECT(routine));
        Py_INCREF(result);

        g_object_unref(G_OBJECT(routine));

    }
    else
    {
        result = Py_None;
        Py_INCREF(result);
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_compiler_demangler_type(void)
{
    static PyMethodDef py_comp_demangler_methods[] = {
        {
            "decode_type", py_compiler_demangler_decode_type,
            METH_VARARGS,
            "decode_type(self, desc/)\n--\n\nDemangle a type definition from its string mangled description."
        },
        {
            "decode_routine", py_compiler_demangler_decode_routine,
            METH_VARARGS,
            "decode_routine(self, desc/)\n--\n\nDemangle a routine definition from its string mangled description."
        },
        { NULL }
    };

    static PyGetSetDef py_comp_demangler_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_comp_demangler_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.mangling.CompDemangler",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT,

        .tp_doc         = "PyChrysalide generic demangler",

        .tp_methods     = py_comp_demangler_methods,
        .tp_getset      = py_comp_demangler_getseters,

    };

    return &py_comp_demangler_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.mangling.DexDemangler'.*
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_compiler_demangler(PyObject *module)
{
    PyTypeObject *py_comp_demangler_type;   /* Type Python 'CompDemangler' */
    PyObject *dict;                         /* Dictionnaire du module      */

    py_comp_demangler_type = get_python_compiler_demangler_type();

    dict = PyModule_GetDict(module);

    if (!register_class_for_pygobject(dict, G_TYPE_COMP_DEMANGLER, py_comp_demangler_type, &PyGObject_Type))
        return false;

    return true;

}
